@@ -1,31 +1,30 @@
 # -*- test-case-name: twisted.conch.test.test_cftp -*-
-#
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-#
-# $Id: cftp.py,v 1.65 2004/03/11 00:29:14 z3p Exp $
+"""
+Implementation module for the I{cftp} command.
+"""
 
-#""" Implementation module for the `cftp` command.
-#"""
-from twisted.conch.client import agent, connect, default, options
-from twisted.conch.error import ConchError
+import os, sys, getpass, struct, tty, fcntl, stat
+import fnmatch, pwd, glob
+
+from twisted.conch.client import connect, default, options
 from twisted.conch.ssh import connection, common
 from twisted.conch.ssh import channel, filetransfer
 from twisted.protocols import basic
 from twisted.internet import reactor, stdio, defer, utils
 from twisted.python import log, usage, failure
 
-import os, sys, getpass, struct, tty, fcntl, base64, signal, stat, errno
-import fnmatch, pwd, time, glob
-
 class ClientOptions(options.ConchOptions):
-    
+
     synopsis = """Usage:   cftp [options] [user@]host
          cftp [options] [user@]host[:dir[/]]
          cftp [options] [user@]host[:file [localfile]]
 """
-    
+    longdesc = ("cftp is a client for logging into a remote machine and "
+                "executing commands to send and receive file information")
+
     optParameters = [
                     ['buffersize', 'B', 32768, 'Size of the buffer to use for sending/receiving.'],
                     ['batchfile', 'b', None, 'File to read commands from, or \'-\' for stdin.'],
@@ -44,7 +43,7 @@ class ClientOptions(options.ConchOptions):
             host, self['remotePath'] = host.split(':', 1)
             self['remotePath'].rstrip('/')
         self['host'] = host
-        self['localPath'] = localPath 
+        self['localPath'] = localPath
 
 def run():
 #    import hotshot
@@ -73,7 +72,6 @@ def run():
 #    prof.close()
 
 def handleError():
-    from twisted.python import failure
     global exitStatus
     exitStatus = 2
     try:
@@ -88,7 +86,7 @@ def doConnect(options):
         options['user'], options['host'] = options['host'].split('@',1)
     host = options['host']
     if not options['user']:
-        options['user'] = getpass.getuser() 
+        options['user'] = getpass.getuser()
     if not options['port']:
         options['port'] = 22
     else:
@@ -128,9 +126,13 @@ class FileWrapper:
 
 class StdioClient(basic.LineReceiver):
 
+    _pwd = pwd
+
     ps = 'cftp> '
     delimiter = '\n'
 
+    reactor = reactor
+
     def __init__(self, client, f = None):
         self.client = client
         self.currentDirectory = ''
@@ -157,6 +159,13 @@ class StdioClient(basic.LineReceiver):
             line = line[1:]
         else:
             self.ignoreErrors = 0
+        d = self._dispatchCommand(line)
+        if d is not None:
+            d.addCallback(self._cbCommand)
+            d.addErrback(self._ebCommand)
+
+
+    def _dispatchCommand(self, line):
         if ' ' in line:
             command, rest = line.split(' ', 1)
             rest = rest.lstrip()
@@ -170,9 +179,7 @@ class StdioClient(basic.LineReceiver):
             log.msg('looking up cmd %s' % command)
             f = getattr(self, 'cmd_%s' % command, None)
         if f is not None:
-            d = defer.maybeDeferred(f, rest)
-            d.addCallback(self._cbCommand)
-            d.addErrback(self._ebCommand)
+            return defer.maybeDeferred(f, rest)
         else:
             self._ebCommand(failure.Failure(NotImplementedError(
                 "No command called `%s'" % command)))
@@ -184,7 +191,7 @@ class StdioClient(basic.LineReceiver):
         if e == NotImplementedError:
             self.transport.write(self.cmd_HELP(''))
         elif e == filetransfer.SFTPError:
-            self.transport.write("remote error %i: %s\n" % 
+            self.transport.write("remote error %i: %s\n" %
                     (f.value.code, f.value.message))
         elif e in (OSError, IOError):
             self.transport.write("local error %i: %s\n" %
@@ -242,7 +249,7 @@ class StdioClient(basic.LineReceiver):
         d = self.client.getAttrs(path)
         d.addCallback(self._cbSetUsrGrp, path, grp=grp)
         return d
-    
+
     def cmd_CHMOD(self, rest):
         mod, rest = rest.split(None, 1)
         path, rest = self._getFilename(rest)
@@ -250,7 +257,7 @@ class StdioClient(basic.LineReceiver):
         d = self.client.setAttrs(path, {'permissions':mod})
         d.addCallback(_ignore)
         return d
-    
+
     def cmd_CHOWN(self, rest):
         usr, rest = rest.split(None, 1)
         path, rest = self._getFilename(rest)
@@ -258,7 +265,7 @@ class StdioClient(basic.LineReceiver):
         d = self.client.getAttrs(path)
         d.addCallback(self._cbSetUsrGrp, path, usr=usr)
         return d
-    
+
     def _cbSetUsrGrp(self, attrs, path, usr=None, grp=None):
         new = {}
         new['uid'] = (usr is not None) and usr or attrs['uid']
@@ -332,8 +339,8 @@ class StdioClient(basic.LineReceiver):
         rf.total = 0.0
         dList = []
         chunks = []
-        startTime = time.time()
-        for i in range(numRequests):            
+        startTime = self.reactor.seconds()
+        for i in range(numRequests):
             d = self._cbGetRead('', rf, lf, chunks, 0, bufferSize, startTime)
             dList.append(d)
         dl = defer.DeferredList(dList, fireOnOneErrback=1)
@@ -353,7 +360,7 @@ class StdioClient(basic.LineReceiver):
         bufSize = int(self.client.transport.conn.options['buffersize'])
         chunks.append((end, end + bufSize))
         return (end, bufSize)
-   
+
     def _cbGetRead(self, data, rf, lf, chunks, start, size, startTime):
         if data and isinstance(data, failure.Failure):
             log.msg('get read err: %s' % data)
@@ -367,14 +374,14 @@ class StdioClient(basic.LineReceiver):
             lf.seek(start)
             lf.write(data)
             if len(data) != size:
-                log.msg('got less than we asked for: %i < %i' % 
+                log.msg('got less than we asked for: %i < %i' %
                         (len(data), size))
                 i = chunks.index((start, start + size))
                 del chunks[i]
                 chunks.insert(i, (start, start + len(data)))
             rf.total += len(data)
         if self.useProgressBar:
-            self._printProgessBar(rf, startTime)
+            self._printProgressBar(rf, startTime)
         chunk = self._getNextChunk(chunks)
         if not chunk:
             return
@@ -392,7 +399,7 @@ class StdioClient(basic.LineReceiver):
         if self.useProgressBar:
             self.transport.write('\n')
         return "Transferred %s to %s" % (rf.name, lf.name)
-   
+
     def cmd_PUT(self, rest):
         local, rest = self._getFilename(rest)
         if '*' in local or '?' in local: # wildcard
@@ -412,7 +419,8 @@ class StdioClient(basic.LineReceiver):
             remote = os.path.split(local)[1]
         lf = file(local, 'r')
         path = os.path.join(self.currentDirectory, remote)
-        d = self.client.openFile(path, filetransfer.FXF_WRITE|filetransfer.FXF_CREAT, {})
+        flags = filetransfer.FXF_WRITE|filetransfer.FXF_CREAT|filetransfer.FXF_TRUNC
+        d = self.client.openFile(path, flags, {})
         d.addCallback(self._cbPutOpenFile, lf)
         d.addErrback(self._ebCloseLf, lf)
         return d
@@ -431,7 +439,7 @@ class StdioClient(basic.LineReceiver):
                 self.transport.write('\n')
         f = None
         while files and not f:
-            try: 
+            try:
                 f = files.pop(0)
                 lf = file(f, 'r')
             except:
@@ -442,7 +450,8 @@ class StdioClient(basic.LineReceiver):
         name = os.path.split(f)[1]
         remote = os.path.join(self.currentDirectory, path, name)
         log.msg((name, remote, path))
-        d = self.client.openFile(remote, filetransfer.FXF_WRITE|filetransfer.FXF_CREAT, {})
+        flags = filetransfer.FXF_WRITE|filetransfer.FXF_CREAT|filetransfer.FXF_TRUNC
+        d = self.client.openFile(remote, flags, {})
         d.addCallback(self._cbPutOpenFile, lf)
         d.addErrback(self._ebCloseLf, lf)
         d.addBoth(self._cbPutMultipleNext, files, path)
@@ -454,7 +463,7 @@ class StdioClient(basic.LineReceiver):
             lf = FileWrapper(lf)
         dList = []
         chunks = []
-        startTime = time.time()
+        startTime = self.reactor.seconds()
         for i in range(numRequests):
             d = self._cbPutWrite(None, rf, lf, chunks, startTime)
             if d:
@@ -470,7 +479,7 @@ class StdioClient(basic.LineReceiver):
         data = lf.read(size)
         if self.useProgressBar:
             lf.total += len(data)
-            self._printProgessBar(lf, startTime)
+            self._printProgressBar(lf, startTime)
         if data:
             d = rf.writeChunk(start, data)
             d.addCallback(self._cbPutWrite, rf, lf, chunks, startTime)
@@ -570,7 +579,7 @@ class StdioClient(basic.LineReceiver):
 
     def cmd_VERSION(self, ignored):
         return "SFTP version %i" % self.client.version
-    
+
     def cmd_HELP(self, ignored):
         return """Available commands:
 cd path                         Change remote directory to 'path'.
@@ -609,8 +618,13 @@ version                         Print the SFTP version.
         return "%ssing progess bar." % (self.useProgressBar and "U" or "Not u")
 
     def cmd_EXEC(self, rest):
-        shell = pwd.getpwnam(getpass.getuser())[6]
-        print repr(rest)
+        """
+        Run C{rest} using the user's shell (or /bin/sh if they do not have
+        one).
+        """
+        shell = self._pwd.getpwnam(getpass.getuser())[6]
+        if not shell:
+            shell = '/bin/sh'
         if rest:
             cmds = ['-c', rest]
             return utils.getProcessOutput(shell, cmds, errortoo=1)
@@ -627,10 +641,10 @@ version                         Print the SFTP version.
         else:
             glob = 0
         if tail and not glob: # could be file or directory
-           # try directory first
-           d = self.client.openDirectory(fullPath)
-           d.addCallback(self._cbOpenList, '')
-           d.addErrback(self._ebNotADirectory, head, tail)
+            # try directory first
+            d = self.client.openDirectory(fullPath)
+            d.addCallback(self._cbOpenList, '')
+            d.addErrback(self._ebNotADirectory, head, tail)
         else:
             d = self.client.openDirectory(head)
             d.addCallback(self._cbOpenList, tail)
@@ -666,11 +680,11 @@ version                         Print the SFTP version.
         # from http://mail.python.org/pipermail/python-list/1999-December/018395.html
         _abbrevs = [
             (1<<50L, 'PB'),
-            (1<<40L, 'TB'), 
-            (1<<30L, 'GB'), 
-            (1<<20L, 'MB'), 
-            (1<<10L, 'kb'),
-            (1, '')
+            (1<<40L, 'TB'),
+            (1<<30L, 'GB'),
+            (1<<20L, 'MB'),
+            (1<<10L, 'kB'),
+            (1, 'B')
             ]
 
         for factor, suffix in _abbrevs:
@@ -690,24 +704,43 @@ version                         Print the SFTP version.
             t -= (60 * mins)
             return "%02i:%02i" % (mins, t)
 
-    def _printProgessBar(self, f, startTime):
-        diff = time.time() - startTime
+
+    def _printProgressBar(self, f, startTime):
+        """
+        Update a console progress bar on this L{StdioClient}'s transport, based
+        on the difference between the start time of the operation and the
+        current time according to the reactor, and appropriate to the size of
+        the console window.
+
+        @param f: a wrapper around the file which is being written or read
+        @type f: L{FileWrapper}
+
+        @param startTime: The time at which the operation being tracked began.
+        @type startTime: C{float}
+        """
+        diff = self.reactor.seconds() - startTime
         total = f.total
         try:
-            winSize = struct.unpack('4H', 
+            winSize = struct.unpack('4H',
                 fcntl.ioctl(0, tty.TIOCGWINSZ, '12345679'))
         except IOError:
             winSize = [None, 80]
-        speed = total/diff
+        if diff == 0.0:
+            speed = 0.0
+        else:
+            speed = total / diff
         if speed:
             timeLeft = (f.size - total) / speed
         else:
             timeLeft = 0
         front = f.name
-        back = '%3i%% %s %sps %s ' % ((total/f.size)*100, self._abbrevSize(total),
-                self._abbrevSize(total/diff), self._abbrevTime(timeLeft))
+        back = '%3i%% %s %sps %s ' % ((total / f.size) * 100,
+                                      self._abbrevSize(total),
+                                      self._abbrevSize(speed),
+                                      self._abbrevTime(timeLeft))
         spaces = (winSize[1] - (len(front) + len(back) + 1)) * ' '
-        self.transport.write('\r%s%s%s' % (front, spaces, back)) 
+        self.transport.write('\r%s%s%s' % (front, spaces, back))
+
 
     def _getFilename(self, line):
         line.lstrip()
@@ -777,7 +810,7 @@ class SSHSession(channel.SSHChannel):
     def eofReceived(self):
         log.msg('got eof')
         self.stdio.closeStdin()
-    
+
     def closeReceived(self):
         log.msg('remote side closed %s' % self)
         self.conn.sendClose(self)
