@@ -1,4 +1,5 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.conch.test.test_scripts -*-
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 #
@@ -204,11 +205,13 @@ class GeneralOptions(usage.Options):
     #zsh_altArgDescr = {"foo":"use this description for foo instead"}
     #zsh_multiUse = ["foo", "bar"]
     zsh_mutuallyExclusive = [("tty", "notty")]
-#    zsh_actions = {"cipher":"(%s)" % " ".join(SSHClientTransport.supportedCiphers),
-#                   "macs":"(%s)" % " ".join(SSHClientTransport.supportedMACs)}
+    zsh_actions = {"cipher":"(%s)" % " ".join(transport.SSHClientTransport.supportedCiphers),
+                   "macs":"(%s)" % " ".join(transport.SSHClientTransport.supportedMACs)}
     zsh_actionDescr = {"localforward":"listen-port:host:port",
                        "remoteforward":"listen-port:host:port"}
-    zsh_extras = ["1:host | user@host: ", "*:command: "]
+    # user, host, or user@host completion similar to zsh's ssh completion
+    zsh_extras = ['1:host | user@host:{_ssh;if compset -P "*@"; then _wanted hosts expl "remote host name" _ssh_hosts && ret=0 elif compset -S "@*"; then _wanted users expl "login name" _ssh_users -S "" && ret=0 else if (( $+opt_args[-l] )); then tmp=() else tmp=( "users:login name:_ssh_users -qS@" ) fi; _alternative "hosts:remote host name:_ssh_hosts" "$tmp[@]" && ret=0 fi}',
+                  '*:command: ']
 
     identitys = []
     localForwards = []
@@ -248,7 +251,7 @@ frame = None
 
 def deferredAskFrame(question, echo):
     if frame.callback:
-        raise "can't ask 2 questions at once!"
+        raise ValueError("can't ask 2 questions at once!")
     d = defer.Deferred()
     resp = []
     def gotChar(ch, resp=resp):
@@ -427,7 +430,7 @@ class SSHUserAuthClient(userauth.SSHUserAuthClient):
         if not os.path.exists(file):
             return
         try:
-            return keys.getPublicKeyString(file) 
+            return keys.Key.fromFile(file).blob() 
         except:
             return self.getPublicKey() # try again
     
@@ -436,7 +439,7 @@ class SSHUserAuthClient(userauth.SSHUserAuthClient):
         if not os.path.exists(file):
             return None
         try:
-            return defer.succeed(keys.getPrivateKeyObject(file))
+            return defer.succeed(keys.Key.fromFile(file).keyObject)
         except keys.BadKeyError, e:
             if e.args[0] == 'encrypted key with no password':
                 prompt = "Enter passphrase for key '%s': " % \
@@ -445,7 +448,7 @@ class SSHUserAuthClient(userauth.SSHUserAuthClient):
     def _cbGetPrivateKey(self, ans, count):
         file = os.path.expanduser(self.usedFiles[-1])
         try:
-            return keys.getPrivateKeyObject(file, password = ans)
+            return keys.Key.fromFile(file, password = ans).keyObject
         except keys.BadKeyError:
             if count == 2:
                 raise
