@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.conch.test.test_insults -*-
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 from twisted.trial import unittest
@@ -458,3 +458,39 @@ class ServerProtocolOutputTests(unittest.TestCase):
         protocol.makeConnection(transport)
         protocol.nextLine()
         self.assertEqual(transport.value(), "\r\n")
+
+
+
+class Deprecations(unittest.TestCase):
+    """
+    Tests to ensure deprecation of L{insults.colors} and L{insults.client}
+    """
+
+    def ensureDeprecated(self, message):
+        """
+        Ensures that the correct deprecation warning was issued.
+        """
+        warnings = self.flushWarnings()
+        self.assertIdentical(warnings[0]['category'], DeprecationWarning)
+        self.assertEquals(warnings[0]['message'], message)
+        self.assertEquals(len(warnings), 1)
+
+
+    def test_colors(self):
+        """
+        The L{insults.colors} module is deprecated
+        """
+        from twisted.conch.insults import colors
+        self.ensureDeprecated("twisted.conch.insults.colors was deprecated "
+                              "in Twisted 10.1.0: Please use "
+                              "twisted.conch.insults.helper instead.")
+
+
+    def test_client(self):
+        """
+        The L{insults.client} module is deprecated
+        """
+        from twisted.conch.insults import client
+        self.ensureDeprecated("twisted.conch.insults.client was deprecated "
+                              "in Twisted 10.1.0: Please use "
+                              "twisted.conch.insults.insults instead.")
