@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -11,6 +11,7 @@ import os, signal, sys, struct
 
 from zope.interface import implements
 
+from twisted.internet.address import IPv4Address
 from twisted.internet.error import ProcessTerminated, ProcessDone
 from twisted.python.failure import Failure
 from twisted.conch.ssh import common, session, connection
@@ -361,7 +362,7 @@ class StubConnection(object):
     """
 
 
-    def __init__(self):
+    def __init__(self, transport=None):
         """
         Initialize our instance variables.
         """
@@ -370,6 +371,7 @@ class StubConnection(object):
         self.requests = {}
         self.eofs = {}
         self.closes = {}
+        self.transport = transport
 
 
     def logPrefix(self):
@@ -418,9 +420,6 @@ class StubConnection(object):
 
 
 
-
-
-
 class StubTransport:
     """
     A stub transport which records the data written.
@@ -436,6 +435,20 @@ class StubTransport:
     close = False
 
 
+    def getPeer(self):
+        """
+        Return an arbitrary L{IAddress}.
+        """
+        return IPv4Address('TCP', 'remotehost', '8888')
+
+
+    def getHost(self):
+        """
+        Return an arbitrary L{IAddress}.
+        """
+        return IPv4Address('TCP', 'localhost', '9999')
+
+
     def write(self, data):
         """
         Record data in the buffer.
@@ -1002,9 +1015,10 @@ class SSHSessionProcessProtocolTestCase(unittest.TestCase):
     """
 
     def setUp(self):
-        self.session = session.SSHSession(
-            conn=StubConnection(), remoteWindow=500, remoteMaxPacket=100)
         self.transport = StubTransport()
+        self.session = session.SSHSession(
+            conn=StubConnection(self.transport), remoteWindow=500,
+            remoteMaxPacket=100)
         self.pp = session.SSHSessionProcessProtocol(self.session)
         self.pp.makeConnection(self.transport)
 
@@ -1033,6 +1047,24 @@ class SSHSessionProcessProtocolTestCase(unittest.TestCase):
         self.assertEquals(self.pp.session, self.session)
 
 
+    def test_getHost(self):
+        """
+        SSHSessionProcessProtocol.getHost() just delegates to its
+        session.conn.transport.getHost().
+        """
+        self.assertEquals(
+            self.session.conn.transport.getHost(), self.pp.getHost())
+
+
+    def test_getPeer(self):
+        """
+        SSHSessionProcessProtocol.getPeer() just delegates to its
+        session.conn.transport.getPeer().
+        """
+        self.assertEquals(
+            self.session.conn.transport.getPeer(), self.pp.getPeer())
+
+
     def test_connectionMade(self):
         """
         SSHSessionProcessProtocol.connectionMade() should check if there's a
@@ -1113,11 +1145,25 @@ class SSHSessionProcessProtocolTestCase(unittest.TestCase):
                 [(1, 'test data')])
 
 
-    def test_inConnectionLost(self):
+    def test_outConnectionLost(self):
         """
-        When inConnectionLost is called, it should send an EOF message,
+        When outConnectionLost and errConnectionLost are both called, we should
+        send an EOF message.
         """
-        self.pp.inConnectionLost()
+        self.pp.outConnectionLost()
+        self.assertFalse(self.session in self.session.conn.eofs)
+        self.pp.errConnectionLost()
+        self.assertTrue(self.session.conn.eofs[self.session])
+
+
+    def test_errConnectionLost(self):
+        """
+        Make sure reverse ordering of events in test_outConnectionLost also
+        sends EOF.
+        """
+        self.pp.errConnectionLost()
+        self.assertFalse(self.session in self.session.conn.eofs)
+        self.pp.outConnectionLost()
         self.assertTrue(self.session.conn.eofs[self.session])
 
 
